# -*- coding: utf-8 -*-
import citrusleaf as cl
from helpers import convertor
from helpers import common
from helpers.caching import CachedMethod
from constants import *
from helpers.info_dict import InfoDict
from datetime import datetime


@CachedMethod()
def get_node_id(ip, port):

    return get_node_info(ip, port).get("node", None)


def is_node_available(ip, port):

    if get_node_info(ip, port) == {}:
        return False
    return True


def get_node_build(ip, port):

    node_build_id = get_node_info(ip, port).get("build", None)
    return node_build_id


@CachedMethod()
def is_xdr_enabled(ip, xdr_port):

    stats = cl.citrusleaf_info(ip, xdr_port, "statistics")
    if stats == UNABLE_TO_CONNECT:
        return False
    return True


@CachedMethod(60)
def get_self_address(ip, port):

    node_address = cl.citrusleaf_info(ip, port, "service")
    if node_address == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    return node_address.split(";")[0]  # multiple_interfaces_on_node


def get_cluster_nodes(ip, port):

    services = get_node_info(ip, port).get("services", UNABLE_TO_CONNECT)
    service = get_self_address(ip, port)
    if services == UNABLE_TO_CONNECT or service == UNABLE_TO_CONNECT:
        return set()

    nodes = convertor.text_to_list(services)
    nodes.append(service.split(";")[0])

    return nodes


def get_node_statistics(ip, port):

    statistics = get_node_info(ip, port, clear_cache=True).get("statistics", None)
    if statistics is None:
        return InfoDict(node_status="off")
    statistics = convertor.text_to_list(statistics)
    statistics = convertor.list_to_dict(statistics)
    statistics['timestamp'] = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    return statistics


@CachedMethod(300)
def get_node_info(ip, port):

    info = cl.citrusleaf_info(ip, port)
    if info == UNABLE_TO_CONNECT:
        return {}
    return info


@CachedMethod()
def get_node_configuration(ip, port):

    configuration = cl.citrusleaf_info(ip, port, "get-config:")
    if configuration == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    configuration = convertor.text_to_list(configuration)
    return convertor.list_to_dict(configuration)


def set_node_configuration(ip, port, parameter, value):

    arg = "set-config:context=service;%s=%s" % (parameter, value)
    if cl.citrusleaf_info(ip, port, arg) == 'ok':
        return True
    return False


def get_xdr_statistics(ip, port):

    statistics = get_xdr_info(ip, port).get("statistics", None)
    if statistics is None:
        return InfoDict(node_status="off")
    statistics = convertor.text_to_list(statistics)
    statistics = convertor.list_to_dict(statistics)
    statistics['timestamp'] = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    return statistics


@CachedMethod()
def get_xdr_info(ip, port):

    info = cl.citrusleaf_info(ip, port)
    if info == UNABLE_TO_CONNECT:
        return {}
    return info


def set_xdr_configuration(ip, port, parameter, value):

    arg = "set-config:context=xdr;%s=%s" % (parameter, value)
    if cl.citrusleaf_info(ip, port, arg) == 'ok':
        return True
    return False


@CachedMethod()
def get_node_bins_info(ip, port):

    bins_info = dict()
    temp = cl.citrusleaf_info(ip, port, "bins")
    if temp == UNABLE_TO_CONNECT:
        return None

    for row in convertor.text_to_list(temp):
        if row:
            namespace, bin_info = convertor.text_to_list(row, ":")
            temp = convertor.text_to_list(bin_info, ",")
            bins_info[namespace] = convertor.list_to_dict(temp[:2])
            bins_info[namespace]["bins"] = temp[2:]

    return bins_info


def configure_node_latency(ip, port):

    result = cl.citrusleaf_info(ip, port, "latency:")
    if result != UNABLE_TO_CONNECT and "error" in str(result):
        if cl.citrusleaf_info(ip, port, "hist-track-start:back=3600;slice=10;") == "ok":
            return True
    return False


@CachedMethod()
def get_node_latency(ip, port):

    latency = dict()
    result = cl.citrusleaf_info(ip, port, "latency:")

    if result == UNABLE_TO_CONNECT or "error" in result:
        return {}

    rows = convertor.text_to_list(result)
    for i in range(0, len(rows), 2):

        if not rows[i]:
            continue
        ind = rows[i].index(':')
        op = rows[i][:ind]
        if op == "writes_reply":
            continue
        rows[i] = rows[i][ind + 1:]

        keys = convertor.text_to_list(rows[i], ",")
        values = convertor.text_to_list(rows[i + 1], ",")

        keys[0] = keys[0].split("-")[0]
        ops_per_sec = float(values[1])

        latency[op] = {'timestamp': common.time_average(values[0], keys[0]),
            'data': [],
            'ops/sec': ops_per_sec
            }

        less_than_1 = 0.0 if ops_per_sec == 0 else 100.0

        previous_key = ""
        previous_value = 0.0
        for key, value in zip(list(reversed(keys))[:len(keys)-2], list(reversed(values))[:len(keys)-2]):
            pct = float(value) - previous_value
            previous_value = float(value)
            original_key = key
            if previous_key != "":
                key = ">" + key[1:] + " to &#x2264;" + previous_key[1:]
            previous_key = original_key
            less_than_1 -= pct
            value = ops_per_sec * pct / 100
            latency[op]['data'] = [{key: dict(value=value, pct=pct)}] + latency[op]['data']

        less_than_1_value = ops_per_sec * less_than_1 / 100
        latency[op]['data'] = [{"&#x2264;1ms": dict(value=less_than_1_value, pct=less_than_1)}] + latency[op]['data']
    return latency


@CachedMethod()
def get_namespaces(ip, port):

    namespaces = cl.citrusleaf_info(ip, port, "namespaces")
    if namespaces == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    return convertor.text_to_set(namespaces)


@CachedMethod()
def get_namespace_statistics(namespace, ip, port):

    stats = cl.citrusleaf_info(ip, port, "namespace/%s" % namespace)
    if stats == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    stats = convertor.text_to_list(stats)
    return convertor.list_to_dict(stats)


def set_namespace_configuration(namespace, ip, port, parameter, value):

    arg = "set-config:context=namespace;id=%s;%s=%s" % (namespace, parameter, value)
    if cl.citrusleaf_info(ip, port, arg) == 'ok':
        return True
    return False


@CachedMethod()
def get_sets_info(ip, port):

    sets_info = dict()
    result = cl.citrusleaf_info(ip, port, "sets")

    if result == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    if result:
        for row in convertor.text_to_list(result.strip(), ";"):
            if not row:
                continue

            details = convertor.list_to_dict(
                convertor.text_to_list(row, ":"), "=")
            for key in list(details.keys()):  # For Aerospike 3.0.22+
                if key.startswith('set-'):
                    new_key = key.replace('set-', '')
                    details[new_key] = details.pop(key)

            namespace = sets_info.get(details['ns_name'], [])
            namespace.append(details)
            sets_info[details['ns_name']] = namespace

    return sets_info


@CachedMethod()
def get_dc_statistics(ip, port, dc):

    statistics = cl.citrusleaf_info(ip, port, "dc/%s" % dc)
    if statistics == UNABLE_TO_CONNECT:
        return InfoDict()
    statistics = convertor.text_to_list(statistics)
    return convertor.list_to_dict(statistics)


@CachedMethod()
def get_dc_names(ip, port):

    names = cl.citrusleaf_info(ip, port, "dcs")
    if names == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    return convertor.text_to_list(names)


@CachedMethod()
def get_udf_list(ip, port):

    lst = cl.citrusleaf_info(ip, port, "udf-list")
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    udfs = []
    for record in convertor.text_to_list(lst, ";"):
        if record != '':
            udfs.append(convertor.list_to_dict(
                convertor.text_to_list(record, ",")))

    return udfs


@CachedMethod()
def get_scan_jobs(ip, port):

    lst = cl.citrusleaf_info(ip, port, "scan-list")
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans":
        return []

    jobs = []
    for record in convertor.text_to_list(lst):
        jobs.append(convertor.list_to_dict(convertor.text_to_list(record, ":")))

    return jobs


@CachedMethod()
def get_job_details1(ip, port):

    lst = cl.citrusleaf_info(ip, port, "mon:cmd=getall-job")
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans" or lst == "" or lst == "error":
        return []

    jobs = {}
    keys = ["trid", "cpu", "mem", "start_time", "run_time", "disk_io_bytes", "net_io_bytes", "priority"]
    for record in lst[1:-1].split("]["):
        details = convertor.text_to_list(record, "|")
        details = dict(zip(keys, details))
        details["address"] = "%s:%s" % (ip, port)
        jobs[details["trid"]] = details

    return jobs


@CachedMethod()
def get_job_details(ip, port):

    lst = cl.citrusleaf_info(ip, port, "jobs:")
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans" or lst == "" or lst == "error":
        return []

    jobs = {}
    for record in convertor.text_to_list(lst.strip(), ";"):
        details = convertor.list_to_dict(
            convertor.text_to_list(record, ":"), "=")
        details["address"] = "%s:%s" % (ip, port)
        jobs[details["trid"]] = details

    return jobs


@CachedMethod()
def get_index_stats(ip, port, namespace, index):

    command = "sindex/%s/%s" % (str(namespace), str(index))
    stats = cl.citrusleaf_info(ip, port, command)
    if stats == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    elif stats == "ns_type=unknown":
        return InfoDict(namespace_found=False)
    elif stats == "Invalid StatsFAIL:201:NO INDEX":
        return InfoDict(index_found=False)
    stats = convertor.text_to_list(stats)
    return convertor.list_to_dict(stats)


@CachedMethod()
def get_indexes(ip, port):

    output = cl.citrusleaf_info(ip, port, "sindex")
    if output == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    indexes = {}
    output = output.replace("sync_state=:", "sync_state=")
    for record in convertor.text_to_list(output):
        details = convertor.list_to_dict(convertor.text_to_list(record, ":"))
        if "ns" not in details:
            continue
        namespace = indexes.get(details["ns"], [])
        namespace.append(details)
        indexes[details["ns"]] = namespace

    return indexes
